"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const redis_1 = require("redis");
const debug_1 = __importDefault(require("debug"));
const log = (0, debug_1.default)("secure-store-redis");
const ALGORITHM = "aes-256-cbc", IV_LENGTH = 16;
class SecureStore {
    constructor(uid, secret, cfg) {
        if (typeof uid !== "string") {
            throw new Error("A uid must be specified");
        }
        else if (typeof secret !== "string") {
            throw new Error("No secret specified");
        }
        else if (secret.length !== 32) {
            throw new Error("Secret must be 32 char string");
        }
        this.uid = uid;
        this.secret = secret;
        this.config = cfg;
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.client) {
                yield this.client.quit();
            }
        });
    }
    disconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.client) {
                yield this.client.disconnect();
            }
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.client) {
                return Promise.resolve();
            }
            else {
                return new Promise((resolve, reject) => {
                    const client = (0, redis_1.createClient)(this.config.redis);
                    client.on("error", (err) => {
                        log("error connecting", err);
                        return reject(err);
                    });
                    client.connect().then(() => {
                        log("connected");
                        this.client = client;
                        resolve();
                    });
                });
            }
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    save(key, data, postfix = "") {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            else if (!data) {
                throw new Error("No data provided, nothing to save");
            }
            postfix = postfix ? ":" + postfix : "";
            if (typeof data === "object") {
                try {
                    data = JSON.stringify(data);
                }
                catch (e) {
                    throw new Error(e);
                }
            }
            yield this.init();
            data = this.encrypt(data);
            const hash = SecureStore.shasum(key);
            return this.client.HSET(this.uid + postfix, hash, data);
        });
    }
    get(key, postfix = "") {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            postfix = postfix ? ":" + postfix : "";
            yield this.init();
            const hash = SecureStore.shasum(key);
            const res = yield this.client.HGET(this.uid + postfix, hash);
            let data;
            if (typeof res === "string") {
                try {
                    data = this.decrypt(res);
                }
                catch (e) {
                    throw new Error(e);
                }
                try {
                    data = JSON.parse(data);
                    // eslint-disable-next-line no-empty
                }
                catch (e) { }
            }
            else {
                data = res;
            }
            return data;
        });
    }
    delete(key, postfix = "") {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof key !== "string") {
                throw new Error("No hash key specified");
            }
            postfix = postfix ? ":" + postfix : "";
            yield this.init();
            const hash = SecureStore.shasum(key);
            return this.client.HDEL(this.uid + postfix, hash);
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    encrypt(data) {
        const iv = (0, crypto_1.randomBytes)(IV_LENGTH);
        const cipher = (0, crypto_1.createCipheriv)(ALGORITHM, Buffer.from(this.secret), iv);
        let encrypted = cipher.update(data);
        encrypted = Buffer.concat([encrypted, cipher.final()]);
        return iv.toString("hex") + ":" + encrypted.toString("hex");
    }
    decrypt(encrypted) {
        const parts = encrypted.split(":");
        const iv = Buffer.from(parts.shift(), "hex");
        const encryptedText = Buffer.from(parts.join(":"), "hex");
        const decipher = (0, crypto_1.createDecipheriv)(ALGORITHM, Buffer.from(this.secret), iv);
        let decrypted = decipher.update(encryptedText);
        decrypted = Buffer.concat([decrypted, decipher.final()]);
        return decrypted.toString();
    }
    static shasum(text) {
        const s = (0, crypto_1.createHash)("sha256");
        s.update(text);
        return s.digest("hex");
    }
}
exports.default = SecureStore;
//# sourceMappingURL=/index.js.map